Attribute VB_Name = "Tools"
Option Explicit

Public Const PI = 3.14159265358979
' Module:     basTools

' Purpose:    Left aligns a string
' Inputs:     Source: String to be left-aligned
'             Length: Length of returned string
' Returns:    Returns a string of length Length, with Source
'             left-aligned and padded with spaces on the right
Public Function LeftAlign(ByVal Source As String, _
        ByVal Length As Integer) As String
    Dim Result As String
    
    Result = Space(Length)
    LSet Result = Source
    LeftAlign = Result
End Function

' Purpose:    Right aligns a string
' Inputs:     Source: String to be Right-aligned
'             Length: Length of returned string
' Returns:    Returns a string of length Length, with Source
'             right-aligned and padded with spaces on the left
Public Function RightAlign(ByVal Source As String, _
        ByVal Length As Integer) As String
    Dim Result As String
    
    Result = Space(Length)
    RSet Result = Source
    RightAlign = Result
End Function

'Purpose:   To calculate the radius for stereographic plot.
Public Function SPlotRadius(ByVal Radius As Double, ByVal Inclination As Double) As Double
    SPlotRadius = Radius * Tan(DegToRad((90 - Inclination) / 2))
End Function

'Purpose:   To convert angles in Degree to Radian.
'           3.14159265358979 / 180 = 0.0174532925199433
Public Function DegToRad(ByVal Theta As Double) As Double
    DegToRad = Theta * 1.74532925199433E-02
End Function

'Purpose:   To convert angles in Radian to Degree.
'           180 / 3.14159265358979 = 57.2957795130824
Public Function RadToDeg(ByVal Theta As Double) As Double
    RadToDeg = Theta * 57.2957795130824
End Function

'Purpose:   To generate correct file name from path and file name
Public Function CorrectFileName(ByVal DriveName As String, ByVal FileName As String) As String
    If Right(DriveName, 1) <> "\" Then
        CorrectFileName = DriveName & "\" & FileName
    Else
        CorrectFileName = DriveName & FileName
    End If
End Function

'Purpose:   To calculate the (x, y, z) coordinate of a vector with length - Radius,
'       Declination - Dec, and Inclination - Inc.
'Dec and Inc in degrees.
'X: Right/E (+), Left/W (-);   Y: Forward (-), Backward (+);   Z: Up (+), Down (-).
Public Sub XYZFromDecInc(ByRef x As Single, ByRef Y As Single, ByRef Z As Single, _
    ByVal Radius As Single, ByVal Dec As Single, ByVal Inc As Single)
    x = Radius * Cos(DegToRad(Inc)) * Sin(DegToRad(Dec))
    Y = Radius * Cos(DegToRad(Inc)) * Cos(DegToRad(Dec))
    Z = Radius * Sin(DegToRad(Inc))
End Sub


'Purpose:   To calculate the Declination and Inclination of a vector (x, y, z).
'DecCalc and IncCalc: calculated Declination and Inclination in degree.
'X = East, Y = North, Z = Down
Public Sub XYZToDecInc(ByVal x As Single, ByVal Y As Single, ByVal Z As Single, _
    ByRef DecCalc As Single, ByRef IncCalc As Single)
    
    Dim RVectorXY As Double     'Length of Vector (x, y, z) on the XY plane
    
    'Calculate the Declination
    If x > 0 Then
        If Y > 0 Then
            DecCalc = RadToDeg(Atn(x / Y))
        ElseIf Y = 0 Then
            DecCalc = 90
        Else    'Y<0
            DecCalc = RadToDeg(Atn(x / Y)) + 180
        End If
    ElseIf x = 0 Then
        If Y >= 0 Then
            DecCalc = 0
        Else    'Y<0
            DecCalc = 180
        End If
    Else    'X<0
        If Y > 0 Then
            DecCalc = RadToDeg(Atn(x / Y)) + 360
        ElseIf Y = 0 Then
            DecCalc = 270
        Else    'Y<0
            DecCalc = RadToDeg(Atn(x / Y)) + 180
        End If
    End If

    'Claculate the Inclination
    RVectorXY = Sqr(x ^ 2 + Y ^ 2)
    If Z <> 0 Then
        If RVectorXY > 0.0000001 Then
            IncCalc = RadToDeg(Atn(Z / RVectorXY))
        Else
            If Z > 0 Then
                IncCalc = 90
            Else
                IncCalc = -90
            End If
        End If
    Else
        IncCalc = 0
    End If
End Sub


'Purpose:   To calculate the radius for Lambert Equal-area projection.
Public Function EQPlotRadius(ByVal Radius As Single, ByVal Inclination As Single) As Single
    EQPlotRadius = Radius * Sqr(1 - Sin(DegToRad(Abs(Inclination))))
End Function

'Purpose:   To find the Maximum and Minimum Values of x and y for Zijderveld Plot
Public Sub FindMaxMin(ByRef MinX As Single, ByRef MaxX As Single, _
    ByRef MinY As Single, ByRef MaxY As Single, ByVal DecX As Single, _
    ByVal DecY As Single, ByVal IncX As Single, ByVal IncY As Single)
    If DecX < MinX Then MinX = DecX
    If DecX > MaxX Then MaxX = DecX
    If IncX < MinX Then MinX = IncX
    If IncX > MaxX Then MaxX = IncX
    If DecY < MinY Then MinY = DecY
    If DecY > MaxY Then MaxY = DecY
    If IncY < MinY Then MinY = IncY
    If IncY > MaxY Then MaxY = IncY
End Sub

'Purpose:   To draw the axes for Zijderveld Projection.
Public Sub DrawZPlotAxes(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, ByVal Theta As Long)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long
    frmSPlot.picZPlot.Line (500, OriginY)-(frmSPlot.picZPlot.ScaleWidth - 500, OriginY)
    frmSPlot.picZPlot.Line (OriginX, 500)-(OriginX, frmSPlot.picZPlot.ScaleHeight - 500)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 500 Then
            frmSPlot.picZPlot.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (frmSPlot.picZPlot.ScaleWidth - 500) Then
            frmSPlot.picZPlot.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 500 Then
            frmSPlot.picZPlot.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (frmSPlot.picZPlot.ScaleHeight - 500) Then
            frmSPlot.picZPlot.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    frmSPlot.picZPlot.CurrentX = OriginX - 300
    frmSPlot.picZPlot.CurrentY = 180
    frmSPlot.picZPlot.Print "Up, W"
    frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 450
    frmSPlot.picZPlot.CurrentY = OriginY - 150
    frmSPlot.picZPlot.Print "N"
    
    TempColor = frmSPlot.picZPlot.ForeColor
    frmSPlot.picZPlot.ForeColor = RGB(0, 0, 255)

    If Theta = -2 Then
        frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 1450
        frmSPlot.picZPlot.CurrentY = OriginY + 50
        frmSPlot.picZPlot.Print "No Projection"
    Else
        frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 2800
        frmSPlot.picZPlot.CurrentY = OriginY + 50
        frmSPlot.picZPlot.Print "Projection Plane: " & Format(Theta, "0") _
            & " degree"
    End If
    
    frmSPlot.picZPlot.ForeColor = TempColor
    
    frmSPlot.picZPlot.CurrentX = OriginX - 260
    frmSPlot.picZPlot.CurrentY = frmSPlot.picZPlot.ScaleHeight - 500
    frmSPlot.picZPlot.Print "Down, E"
    frmSPlot.picZPlot.CurrentX = 200
    frmSPlot.picZPlot.CurrentY = OriginY - 120
    frmSPlot.picZPlot.Print "S"
    frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 2500
    frmSPlot.picZPlot.CurrentY = 40
    frmSPlot.picZPlot.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    'TempColor = frmSPlot.picZPlot.ForeColor
    frmSPlot.picZPlot.ForeColor = RGB(255, 0, 0)
    frmSPlot.picZPlot.CurrentX = 300
    frmSPlot.picZPlot.CurrentY = frmSPlot.picZPlot.ScaleHeight - 250
    frmSPlot.picZPlot.Print "Declination"
    frmSPlot.picZPlot.ForeColor = RGB(0, 0, 255)
    frmSPlot.picZPlot.CurrentX = 1500
    frmSPlot.picZPlot.CurrentY = frmSPlot.picZPlot.ScaleHeight - 250
    frmSPlot.picZPlot.Print "Inclination"
    frmSPlot.picZPlot.ForeColor = TempColor
End Sub

'Purpose:   To draw the axes for Zijderveld Projection.
Public Sub DrawZPlotAxes_Modified(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, ByVal Theta As Long)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long
    frmSPlot.picZPlot.Line (500, OriginY)-(frmSPlot.picZPlot.ScaleWidth - 500, OriginY)
    frmSPlot.picZPlot.Line (OriginX, 500)-(OriginX, frmSPlot.picZPlot.ScaleHeight - 500)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 500 Then
            frmSPlot.picZPlot.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (frmSPlot.picZPlot.ScaleWidth - 500) Then
            frmSPlot.picZPlot.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 500 Then
            frmSPlot.picZPlot.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (frmSPlot.picZPlot.ScaleHeight - 500) Then
            frmSPlot.picZPlot.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    frmSPlot.picZPlot.CurrentX = OriginX - 300
    frmSPlot.picZPlot.CurrentY = 180
    frmSPlot.picZPlot.Print "N, Up"
    frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 450
    frmSPlot.picZPlot.CurrentY = OriginY - 150
    frmSPlot.picZPlot.Print "E"
    
    TempColor = frmSPlot.picZPlot.ForeColor
    frmSPlot.picZPlot.ForeColor = RGB(0, 0, 255)

    If Theta = -2 Then
        frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 1450
        frmSPlot.picZPlot.CurrentY = OriginY + 50
        frmSPlot.picZPlot.Print "No Projection"
    Else
        frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 2800
        frmSPlot.picZPlot.CurrentY = OriginY + 50
        frmSPlot.picZPlot.Print "Projection Plane: " & Format(Theta, "0") _
            & " degree"
    End If
    
    frmSPlot.picZPlot.ForeColor = TempColor

    frmSPlot.picZPlot.CurrentX = OriginX - 260
    frmSPlot.picZPlot.CurrentY = frmSPlot.picZPlot.ScaleHeight - 500
    frmSPlot.picZPlot.Print "S, Down"
    frmSPlot.picZPlot.CurrentX = 200
    frmSPlot.picZPlot.CurrentY = OriginY - 120
    frmSPlot.picZPlot.Print "W"
    frmSPlot.picZPlot.CurrentX = frmSPlot.picZPlot.ScaleWidth - 2500
    frmSPlot.picZPlot.CurrentY = 40
    frmSPlot.picZPlot.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    TempColor = frmSPlot.picZPlot.ForeColor
    frmSPlot.picZPlot.ForeColor = RGB(255, 0, 0)
    frmSPlot.picZPlot.CurrentX = 300
    frmSPlot.picZPlot.CurrentY = frmSPlot.picZPlot.ScaleHeight - 250
    frmSPlot.picZPlot.Print "Declination"
    frmSPlot.picZPlot.ForeColor = RGB(0, 0, 255)
    frmSPlot.picZPlot.CurrentX = 1500
    frmSPlot.picZPlot.CurrentY = frmSPlot.picZPlot.ScaleHeight - 250
    frmSPlot.picZPlot.Print "Inclination"
    frmSPlot.picZPlot.ForeColor = TempColor
End Sub

'Purpose:   To print the axes for Zijderveld Projection.
Public Sub PrintZPlotAxes(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, _
    ByVal TickNum As Integer, ByVal Theta As Long)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long, Length As Single
    
    Length = ScaleTwips * TickNum
    
    Printer.Line (1440, OriginY)-(Length + 2160, OriginY)
    Printer.Line (OriginX, 2160)-(OriginX, Length + 2160)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 1440 Then
            Printer.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (Length + 2160) Then
            Printer.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 2160 Then
            Printer.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (Length + 2160) Then
            Printer.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    Printer.CurrentX = OriginX - 400
    Printer.CurrentY = 1860
    Printer.Print "Up, W"
    Printer.CurrentX = Length + 2220
    Printer.CurrentY = OriginY - 120
    Printer.Print "N"
    If Theta = -2 Then
        Printer.CurrentX = Length + 600
        Printer.CurrentY = OriginY + 50
        Printer.Print "No Projection"
    Else
        Printer.CurrentX = Length - 1600
        Printer.CurrentY = OriginY + 50
        Printer.Print "Projection Plane: " & Format(Theta, "0") _
            & " degree"
    End If
    Printer.CurrentX = OriginX - 460
    Printer.CurrentY = Length + 2220
    Printer.Print "Down, E"
    Printer.CurrentX = 1200
    Printer.CurrentY = OriginY - 120
    Printer.Print "S"
    Printer.CurrentX = 7000
    Printer.CurrentY = Length + 2500
    Printer.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    TempColor = Printer.ForeColor
    Printer.ForeColor = RGB(255, 0, 0)
    Printer.CurrentX = 1440
    Printer.CurrentY = Length + 2500
    Printer.Print "Declination"
    Printer.ForeColor = RGB(0, 0, 255)
    Printer.CurrentX = 3440
    Printer.CurrentY = Length + 2500
    Printer.Print "Inclination"
    Printer.ForeColor = TempColor
End Sub

'Purpose:   To print the axes for Modified Zijderveld Projection.
Public Sub PrintZPlotAxes_Modified(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, _
    ByVal TickNum As Integer, ByVal Theta As Long)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long, Length As Single
    
    Length = ScaleTwips * TickNum
    
    Printer.Line (1440, OriginY)-(Length + 2160, OriginY)
    Printer.Line (OriginX, 2160)-(OriginX, Length + 2160)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 1440 Then
            Printer.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (Length + 2160) Then
            Printer.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 2160 Then
            Printer.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (Length + 2160) Then
            Printer.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    Printer.CurrentX = OriginX - 400
    Printer.CurrentY = 1860
    Printer.Print "N, Up"
    Printer.CurrentX = Length + 2220
    Printer.CurrentY = OriginY - 120
    Printer.Print "E"
    If Theta = -2 Then
        Printer.CurrentX = Length + 600
        Printer.CurrentY = OriginY + 50
        Printer.Print "No Projection"
    Else
        Printer.CurrentX = Length - 1600
        Printer.CurrentY = OriginY + 50
        Printer.Print "Projection Plane: " & Format(Theta, "0") _
            & " degree"
    End If
    Printer.CurrentX = OriginX - 460
    Printer.CurrentY = Length + 2220
    Printer.Print "S, Down"
    Printer.CurrentX = 1200
    Printer.CurrentY = OriginY - 120
    Printer.Print "W"
    Printer.CurrentX = 7000
    Printer.CurrentY = Length + 2500
    Printer.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    TempColor = Printer.ForeColor
    Printer.ForeColor = RGB(255, 0, 0)
    Printer.CurrentX = 1440
    Printer.CurrentY = Length + 2500
    Printer.Print "Declination"
    Printer.ForeColor = RGB(0, 0, 255)
    Printer.CurrentX = 3440
    Printer.CurrentY = Length + 2500
    Printer.Print "Inclination"
    Printer.ForeColor = TempColor
End Sub

'Purpose:   To find range of data for Zijderveld Plot.
Public Sub FindRange(ByRef Range As Single, ByVal MinX As Single, _
    ByVal MaxX As Single, ByVal MinY As Single, ByVal MaxY As Single)
    If (MaxX - MinX) > (MaxY - MinY) Then
        Range = MaxX - MinX
    Else
        Range = MaxY - MinY
    End If
End Sub


'Purpose:   To find the actual length for a tick mark in Zijderveld Plot
Public Function FindScaleLength(ByVal Range As Single) As Single
    Dim N As Integer
    N = 0
    Do Until (Range * (10 ^ N) > 40)
        N = N + 1
    Loop
    If (Int(Range * (10 ^ (N - 1))) + 1) < 10 Then
        FindScaleLength = 5 * 10 ^ (-N)
    ElseIf (Int(Range * (10 ^ (N - 1))) + 1) < 20 Then
        FindScaleLength = 1 * 10 ^ (-N + 1)
    Else
        FindScaleLength = 2 * 10 ^ (-N + 1)
    End If
End Function


'Purpose:   To calculate the angle between two vectors. Used mainly in Fisher analysis.
Public Function ArcCos(ByVal x As Double) As Double
    If x = 1 Or x = -1 Then         '???? X=1, ArcCos=0; X=-1, ArcCos=PI
      ArcCos = 0
    Else
      ArcCos = -1 * Atn(x / Sqr(Abs(-x * x + 1))) + PI / 2     '**  ARCCOS FUNCTION
    End If
End Function


Public Function ArcSin(ByVal x As Double) As Double
    If x = 1 Or x = -1 Then         'Original: ArcSin = PI/2. Changed on March 5, 2002
'    If (Abs(X) - 1) < 0.00001 Then         'Original: ArcSin = PI/2. Changed on March 5, 2002
      ArcSin = x * PI / 2
    Else
      ArcSin = Atn(x / Sqr(Abs(-x * x + 1)))       '**  ARCSIN FUNCTION
    End If
End Function

'Purpose: calculate Declination from cartesian coordinates (X = North, Y = East)
Public Function ArcTan(ByVal x As Double, ByVal Y As Double) As Double
    Dim Z As Double
    
    If x = 0 Then
        If Y < 0 Then Z = -PI / 2
        If Y > 0 Then Z = PI / 2
        If Y = 0 Then Z = 0
    Else
        Z = Atn(Y / x)
        '**  PUT z IN PROPER QUADRANT
        If (x < 0 And Y < 0) Then Z = Z - PI
        If (x < 0 And Y >= 0) Then Z = Z + PI
        If Z < 0 Then Z = 2 * PI + Z
    End If
    ArcTan = Z
End Function

Public Function HyperSin(ByVal x As Double) As Double
    HyperSin = (Exp(x) - Exp(-x)) / 2
End Function

Public Function HyperCos(ByVal x As Double) As Double
    HyperCos = (Exp(x) + Exp(-x)) / 2
End Function

Public Function HyperCoTan(ByVal x As Double) As Double
    HyperCoTan = HyperCos(x) / HyperSin(x)
End Function

'Purpose:   To draw the axes for Zijderveld Projection.
Public Sub PointsZPlotAxes_Modified(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, ByVal Theta As Long, _
    ByVal DecColor As ColorConstants, ByVal IncColor As ColorConstants)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long
    frmPointsPlot.picPointsPlot.Line (500, OriginY)-(frmPointsPlot.picPointsPlot.ScaleWidth - 500, OriginY)
    frmPointsPlot.picPointsPlot.Line (OriginX, 500)-(OriginX, frmPointsPlot.picPointsPlot.ScaleHeight - 500)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 500 Then
            frmPointsPlot.picPointsPlot.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (frmPointsPlot.picPointsPlot.ScaleWidth - 500) Then
            frmPointsPlot.picPointsPlot.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 500 Then
            frmPointsPlot.picPointsPlot.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (frmPointsPlot.picPointsPlot.ScaleHeight - 500) Then
            frmPointsPlot.picPointsPlot.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    frmPointsPlot.picPointsPlot.CurrentX = OriginX - 300
    frmPointsPlot.picPointsPlot.CurrentY = 180
    frmPointsPlot.picPointsPlot.Print "N, Up"
    frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 450
    frmPointsPlot.picPointsPlot.CurrentY = OriginY - 150
    frmPointsPlot.picPointsPlot.Print "E"
    
    TempColor = frmPointsPlot.picPointsPlot.ForeColor
    frmPointsPlot.picPointsPlot.ForeColor = IncColor

    If Theta = -2 Then
        frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 2200
        frmPointsPlot.picPointsPlot.CurrentY = OriginY + 50
        frmPointsPlot.picPointsPlot.Print "Horizontal; No Projection"
    Else
        frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 2400
        frmPointsPlot.picPointsPlot.CurrentY = OriginY + 50
        frmPointsPlot.picPointsPlot.Print "Projection Plane: " & Format(Theta, "0") _
            & " degree"
    End If
    
    frmPointsPlot.picPointsPlot.ForeColor = TempColor

    frmPointsPlot.picPointsPlot.CurrentX = OriginX - 260
    frmPointsPlot.picPointsPlot.CurrentY = frmPointsPlot.picPointsPlot.ScaleHeight - 500
    frmPointsPlot.picPointsPlot.Print "S, Down"
    frmPointsPlot.picPointsPlot.CurrentX = 230
    frmPointsPlot.picPointsPlot.CurrentY = OriginY - 120
    frmPointsPlot.picPointsPlot.Print "W"
    frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 2500
    frmPointsPlot.picPointsPlot.CurrentY = 40
    frmPointsPlot.picPointsPlot.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    TempColor = frmPointsPlot.picPointsPlot.ForeColor
    frmPointsPlot.picPointsPlot.ForeColor = DecColor
    frmPointsPlot.picPointsPlot.CurrentX = 300
    frmPointsPlot.picPointsPlot.CurrentY = frmPointsPlot.picPointsPlot.ScaleHeight - 250
    frmPointsPlot.picPointsPlot.Print "Declination"
    frmPointsPlot.picPointsPlot.ForeColor = IncColor
    frmPointsPlot.picPointsPlot.CurrentX = 1500
    frmPointsPlot.picPointsPlot.CurrentY = frmPointsPlot.picPointsPlot.ScaleHeight - 250
    frmPointsPlot.picPointsPlot.Print "Inclination"
    frmPointsPlot.picPointsPlot.ForeColor = TempColor
End Sub

'Purpose:   To draw the axes for Zijderveld Projection.
Public Sub PointsZPlotAxes(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, ByVal Theta As Long, _
    ByVal DecColor As ColorConstants, ByVal IncColor As ColorConstants)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long
    frmPointsPlot.picPointsPlot.Line (500, OriginY)-(frmPointsPlot.picPointsPlot.ScaleWidth - 500, OriginY)
    frmPointsPlot.picPointsPlot.Line (OriginX, 500)-(OriginX, frmPointsPlot.picPointsPlot.ScaleHeight - 500)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 500 Then
            frmPointsPlot.picPointsPlot.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (frmPointsPlot.picPointsPlot.ScaleWidth - 500) Then
            frmPointsPlot.picPointsPlot.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 500 Then
            frmPointsPlot.picPointsPlot.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (frmPointsPlot.picPointsPlot.ScaleHeight - 500) Then
            frmPointsPlot.picPointsPlot.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    frmPointsPlot.picPointsPlot.CurrentX = OriginX - 300
    frmPointsPlot.picPointsPlot.CurrentY = 180
    frmPointsPlot.picPointsPlot.Print "Up, W"
    frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 450
    frmPointsPlot.picPointsPlot.CurrentY = OriginY - 150
    frmPointsPlot.picPointsPlot.Print "N"
    
    TempColor = frmPointsPlot.picPointsPlot.ForeColor
    frmPointsPlot.picPointsPlot.ForeColor = IncColor

    If Theta = -2 Then
        frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 1250
        frmPointsPlot.picPointsPlot.CurrentY = OriginY + 50
        frmPointsPlot.picPointsPlot.Print "No Projection"
    Else
        frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 2400
        frmPointsPlot.picPointsPlot.CurrentY = OriginY + 50
        frmPointsPlot.picPointsPlot.Print "Projection Plane: " & Format(Theta, "0") _
            & " degree"
    End If
    
    frmPointsPlot.picPointsPlot.ForeColor = TempColor
    
    frmPointsPlot.picPointsPlot.CurrentX = OriginX - 260
    frmPointsPlot.picPointsPlot.CurrentY = frmPointsPlot.picPointsPlot.ScaleHeight - 500
    frmPointsPlot.picPointsPlot.Print "Down, E"
    frmPointsPlot.picPointsPlot.CurrentX = 300
    frmPointsPlot.picPointsPlot.CurrentY = OriginY - 120
    frmPointsPlot.picPointsPlot.Print "S"
    frmPointsPlot.picPointsPlot.CurrentX = frmPointsPlot.picPointsPlot.ScaleWidth - 2500
    frmPointsPlot.picPointsPlot.CurrentY = 40
    frmPointsPlot.picPointsPlot.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    'TempColor = frmPointsPlot.picPointsPlot.ForeColor
    frmPointsPlot.picPointsPlot.ForeColor = DecColor
    frmPointsPlot.picPointsPlot.CurrentX = 300
    frmPointsPlot.picPointsPlot.CurrentY = frmPointsPlot.picPointsPlot.ScaleHeight - 250
    frmPointsPlot.picPointsPlot.Print "Declination"
    frmPointsPlot.picPointsPlot.ForeColor = IncColor
    frmPointsPlot.picPointsPlot.CurrentX = 1500
    frmPointsPlot.picPointsPlot.CurrentY = frmPointsPlot.picPointsPlot.ScaleHeight - 250
    frmPointsPlot.picPointsPlot.Print "Inclination"
    frmPointsPlot.picPointsPlot.ForeColor = TempColor
End Sub

'Purpose:   To print the axes for Zijderveld Projection.
Public Sub PrintPointsZPlotAxes(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, ByVal Theta As Long, _
    ByVal DecColor As ColorConstants, ByVal IncColor As ColorConstants, _
    ByVal TickNum As Integer)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long, Length As Single
    
    Length = ScaleTwips * TickNum
    
    Printer.Line (1440, OriginY)-(Length + 1440, OriginY)
    Printer.Line (OriginX, 2160)-(OriginX, Length + 2160)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 1440 Then
            Printer.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (Length + 1440) Then
            Printer.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 2160 Then
            Printer.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (Length + 2160) Then
            Printer.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    Printer.CurrentX = OriginX - 400
    Printer.CurrentY = 1860
    Printer.Print "Up, W"
    Printer.CurrentX = Length + 1500
    Printer.CurrentY = OriginY - 120
    Printer.Print "N"
    
    TempColor = Printer.ForeColor
    Printer.ForeColor = IncColor

    If Theta = -2 Then
        Printer.CurrentX = Length - 120
        Printer.CurrentY = OriginY + 50
        Printer.Print "No Projection"
    Else
        Printer.CurrentX = Length - 2320
        Printer.CurrentY = OriginY + 50
        Printer.Print "Projection Plane: " & Format(Theta, "0") & " degree"
    End If
    
    Printer.ForeColor = TempColor
    
    Printer.CurrentX = OriginX - 460
    Printer.CurrentY = Length + 2220
    Printer.Print "Down, E"
    Printer.CurrentX = 1200
    Printer.CurrentY = OriginY - 120
    Printer.Print "S"
    Printer.CurrentX = Length - 2200
    Printer.CurrentY = 1500
    Printer.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    TempColor = Printer.ForeColor
    Printer.ForeColor = DecColor
    Printer.CurrentX = 1440
    Printer.CurrentY = Length + 2500
    Printer.Print "Declination"
    Printer.ForeColor = IncColor
    Printer.CurrentX = 3440
    Printer.CurrentY = Length + 2500
    Printer.Print "Inclination"
    Printer.ForeColor = TempColor
End Sub

'Purpose:   To print the axes for Zijderveld Projection.
Public Sub PrintPointsZPlotAxes_Modified(ByVal OriginX As Single, ByVal OriginY As Single, _
    ByVal ScaleTwips As Single, ByVal ScaleLength As Single, ByVal Theta As Long, _
    ByVal DecColor As ColorConstants, ByVal IncColor As ColorConstants, _
    ByVal TickNum As Integer)
    Dim I As Integer
    Dim XTickLT0 As Single, YTickLT0 As Single    'Tick mark positions on x/y axes
    Dim XTickGT0 As Single, YTickGT0 As Single    'Tick mark positions on x/y axes
    Dim TempColor As Long, Length As Single
    
    Length = ScaleTwips * TickNum
    
    Printer.Line (1440, OriginY)-(Length + 1440, OriginY)
    Printer.Line (OriginX, 2160)-(OriginX, Length + 2160)
    
    'Purpose:   Draw tick marks
    For I = 1 To 20
        XTickLT0 = OriginX - I * ScaleTwips
        XTickGT0 = OriginX + I * ScaleTwips
        YTickLT0 = OriginY - I * ScaleTwips
        YTickGT0 = OriginY + I * ScaleTwips
        
        If XTickLT0 > 1440 Then
            Printer.Line (XTickLT0, OriginY - 80)-(XTickLT0, OriginY + 80)
        End If
        If XTickGT0 < (Length + 1440) Then
            Printer.Line (XTickGT0, OriginY - 80)-(XTickGT0, OriginY + 80)
        End If
        If YTickLT0 > 2160 Then
            Printer.Line (OriginX - 80, YTickLT0)-(OriginX + 80, YTickLT0)
        End If
        If YTickGT0 < (Length + 2160) Then
            Printer.Line (OriginX - 80, YTickGT0)-(OriginX + 80, YTickGT0)
        End If
    Next I
    
    'Purpose:   To write the labels for the axis
    Printer.CurrentX = OriginX - 400
    Printer.CurrentY = 1860
    Printer.Print "N, Up"
    Printer.CurrentX = Length + 1500
    Printer.CurrentY = OriginY - 120
    Printer.Print "E"
    
    TempColor = Printer.ForeColor
    Printer.ForeColor = IncColor

    If Theta = -2 Then
        Printer.CurrentX = Length - 120
        Printer.CurrentY = OriginY + 50
        Printer.Print "No Projection"
    Else
        Printer.CurrentX = Length - 2320
        Printer.CurrentY = OriginY + 50
        Printer.Print "Projection Plane: " & Format(Theta, "0") _
            & " degree"
    End If
    
    Printer.ForeColor = TempColor

    Printer.CurrentX = OriginX - 460
    Printer.CurrentY = Length + 2220
    Printer.Print "S, Down"
    Printer.CurrentX = 1200
    Printer.CurrentY = OriginY - 120
    Printer.Print "W"
    Printer.CurrentX = Length - 2200
    Printer.CurrentY = 1500
    Printer.Print "1 Tick Mark = " & Format(ScaleLength, "Scientific")
    
    'Purpose:   To write Declination and Inclination at bottom of Zijderveld Plot.
    TempColor = Printer.ForeColor
    Printer.ForeColor = DecColor
    Printer.CurrentX = 1440
    Printer.CurrentY = Length + 2500
    Printer.Print "Declination"
    Printer.ForeColor = IncColor
    Printer.CurrentX = 3440
    Printer.CurrentY = Length + 2500
    Printer.Print "Inclination"
    Printer.ForeColor = TempColor
End Sub


'To calculate the VGP latitude and longitude from Site latitude, longitude, declination,
'   and inclination. VGPLat and VGPLong in degrees; others in Radian.
Public Function VGPFromSite(ByVal SampleDec As Single, ByVal SampleInc As Single, _
    ByVal SampleLat As Single, ByVal SampleLong As Single, _
    ByRef VGPLat As Single, ByRef VGPLong As Single)
    Dim SampleCo_Lat As Single
    Dim Magnetic_Co_Lat As Single, VGPCo_Lat As Single, DeltaVGPLong As Single
    
    SampleCo_Lat = PI / 2 - SampleLat
    
    Magnetic_Co_Lat = PI / 2 - Atn(Tan(SampleInc) / 2)
    
    VGPCo_Lat = ArcCos(Cos(SampleCo_Lat) * Cos(Magnetic_Co_Lat) + Sin(SampleCo_Lat) * Sin(Magnetic_Co_Lat) * Cos(SampleDec))
    
    VGPLat = RadToDeg(PI / 2 - VGPCo_Lat)
    
    DeltaVGPLong = ArcSin(Sin(Magnetic_Co_Lat) * Sin(SampleDec) / Sin(VGPCo_Lat))
       
    If Cos(Magnetic_Co_Lat) >= Cos(SampleCo_Lat) * Cos(VGPCo_Lat) Then
        VGPLong = RadToDeg(SampleLong + DeltaVGPLong)
    Else
        VGPLong = RadToDeg(SampleLong + PI - DeltaVGPLong)
    End If
End Function
